/**
* @Author: zhangshu
* @email: shiliang.zhang@tuya.com
* @LastEditors: zhangshu
* @file name: air_data_received_handle.h
* @Description: air data received handle
* @Copyright: HANGZHOU TUYA INFORMATION TECHNOLOGY CO.,LTD
* @Company: http://www.tuya.com
* @Date: 2022-11-13
* @LastEditTime: 2022-11-13
*/
#ifndef __AIR_DATA_RECEIVED_HANDLE_H
#define __AIR_DATA_RECEIVED_HANDLE_H

#include "tuya_zigbee_stack.h"
#include "app_common.h"

#ifdef __cplusplus
	extern "C" {
#endif

//air command
//gateway send dp data to zigbee module, zigbee module send to mcu through uart protocol
#define    SERIAL_PROTOCOL_DATA_REQ           0x00
//zigbee module response gateway dp data after obtain response from mcu
#define    SERIAL_PROTOCOL_DATA_RSP           0x01
//zigbee module actively reports dp data to gateway when receive report dp data from mcu
#define    SERIAL_PROTOCOL_DATA_REPORT        0x02
//gateway send dp query to zigbee module, zigbee module send to mcu through uart protocol
#define    SERIAL_PROTOCOL_DATA_QUERY         0x03
//gateway send query specific dp data to zigbee module, zigbee module send to mcu through uart protocol
#define    QUERY_SPECIFIC_DP_CMD              0x08
//gateway send mcu version request to zigbee module, zigbee module send to mcu through uart protocol
#define    SERIAL_PROTOCOL_MCU_OTA_VER_REQ    0x10
//zigbee module response gateway mcu version request after obtain response from mcu
#define    SERIAL_PROTOCOL_MCU_OTA_VER_RSP    0x11
//gateway send ota notify to zigbee module, zigbee module send to mcu through uart potocol
#define    SERIAL_PROTOCOL_MCU_OTA_NOTIFY     0x12
//zigbee module send mcu ota data request to gateway when mcu send mcu ota data request to zigbee module
#define    SERIAL_PROTOCOL_MCU_OTA_DATA_REQ   0x13
//gateway respose mcu ota data request to zigbee module, zigbee module send to mcu throuhg uart protocol
#define    SERIAL_PROTOCOL_MCU_OTA_DATA_RSP   0x14
//zigbee module send mcu ota status to gateway after receive mcu ota result from mcu
#define    SERIAL_PROTOCOL_MCU_OTA_END        0x15
//zigbee module send time request to gateway after receive mcu time sync request
#define    SERIAL_PROTOCOL_TIME_REQ           0x24
//zigbee module send gateway network status request to gateway when receive request from mcu
#define    SERIAL_PROTOCOL_GW_NETWORK_ST_REQ  0x25

#if COMMON_LINK_SCENE
#define    SERIAL_PROTOCOL_DP_SEND_CMD        0x04
#define    SERIAL_PROTOCOL_DP_RESPONSE_CMD    0x05
#endif

/**
* @description: get air message type
* @param  none
* @return: air send message type
*/
AIR_SEND_TYPE_E air_message_type_get(VOID_T);

/**
* @description: group ability set
* @param {type} group ability
* @return: none
*/
VOID_T group_ability_set(UINT8_T type);

/**
 * Af_command_parse_handle
 * Cluster: serial protocol private cluster command parse callback
 * @param {cmd} air command of private cluster
 * @param {mode} delivery mode
 * @param {buff} air data 
 * @param {buff_len} air data len
 * @return operate status
 */
OPERATE_RET air_private_cluster_command_parse_handle(UINT8_T cmd, ZG_DELIVERY_MODE_E mode, UINT8_T *buff, UINT8_T buff_len);

#if COMMON_LINK_SCENE
/**
* @description: load user scene info
* @param[in] switch_id: switch id
* @param[in] *group_id: group id
* @param[in] *scene_id: scene id
* @return: return TRUE when valid, else return FALSE
*/
BOOL_T app_switch_id_valid_check_and_get(UINT8_T switch_id, UINT8_T *group_id, UINT8_T *scene_id);

/**
* @description: clear user scene info
* @param[in] group_id: group id
* @return: none
*/
VOID_T app_user_scene_info_clear(UINT16_T group_id);
#endif

/**
 * @brief air data reveiced handle event init
 *
 * @param[in] none
 * @return none
 */
VOID_T air_data_received_handle_event_init(VOID_T);

#ifdef __cplusplus
}
#endif
#endif

