/**
* @Author: zhangshu
* @email: shiliang.zhang@tuya.com
* @LastEditors: zhangshu
* @file name: air_data_send.h
* @Description: air data send
* @Copyright: HANGZHOU TUYA INFORMATION TECHNOLOGY CO.,LTD
* @Company: http://www.tuya.com
* @Date: 2022-11-13
* @LastEditTime: 2022-11-13
*/
#ifndef __AIR_DATA_SEND_H
#define __AIR_DATA_SEND_H

#include "tuya_zigbee_stack.h"
#include "tal_data_send.h"
#include "app_common.h"

#ifdef __cplusplus
	extern "C" {
#endif

/**
* @description: air data send mcu DP read response to gateway
* @param {* data} receive data ptr
* @param {data_len} receive data length
* @param {seq} seq
* @param {force_broad} force broadcast send message or not
* @return: none
*/
VOID_T air_mcu_DP_response_cmd_send(UINT8_T* data, 
                                               UINT8_T data_len, 
                                               UINT16_T seq, 
                                               BOOL_T force_broad);

/**
* @description: air data send mcu data report to gateway
* @param {data} receive data ptr
* @param {data_len} receive data length
* @param {result} tal send result
* @return: none
*/
VOID_T air_mcu_DP_report_cmd_send(UINT8_T* data, 
                                            UINT8_T data_len, 
                                            TAL_SEND_RESULT_CB result);

/**
* @description: air send mcu version report to gateway
* @param {none}
* @return: none
*/
VOID_T air_mcu_version_report_send(VOID_T);

/**
* @description: serial data response send callback, receive data from mcu and send to gateway
* @param {cmd_id} send data command id
* @param {* data} receive data ptr
* @param {data_len} receive data length
* @param {re_send} resend flag
* @return: none
*/
VOID_T air_mcu_ota_data_request_send(UINT8_T cmd_id, 
                                                 UINT8_T* data, 
                                                 UINT8_T data_len,
                                                 BOOL_T re_send);

/**
* @description: air data send time request to gateway
* @param {* data} receive data ptr
* @param {data_len} receive data length
* @return: none
*/
VOID_T air_time_request_cmd_send(UINT8_T* data, UINT8_T data_len);

/**
* @description: air gateway network status request to gateway
* @param {* data} receive data ptr
* @param {data_len} receive data length
* @return: none
*/
VOID_T air_gateway_network_status_request_send(UINT8_T *data, UINT8_T data_len);

#if COMMON_LINK_SCENE
/**
* @description: air scene add response send
* @param {seq} request seq
* @return: none
*/
VOID_T air_scene_add_response_send(UINT16_T seq);

/**
* @description: air recall scene send
* @param {group_id} group id
* @param {scene_id} scene id
* @return: none
*/
VOID_T air_standard_recall_scene_send(UINT8_T group_id, UINT8_T scene_id);

/**
* @description: air group broadcst standard command send
* @param {groupid} group id
* @param {cluster} cluster id
* @param {cmd} command id
* @param {payload} payload data
* @param {payload_len} payload length
* @return: none
*/
VOID_T air_group_broadcast_standard_cmd_send(UINT16_T groupid, UINT16_T cluster, UINT8_T cmd, UINT8_T * payload, UINT8_T payload_len);

/**
* @description: air group broadcst private command send
* @param {groupid} group id
* @param {payload} payload data
* @param {payload_len} payload length
* @return: none
*/
VOID_T air_group_broadcast_private_cmd_send(UINT16_T groupid, UINT8_T * payload, UINT8_T payload_len);

#endif

/**
 * @brief air send init
 *
 * @param[in] none
 * @return none
 */
VOID_T air_send_init(VOID_T);

#ifdef __cplusplus
}
#endif
#endif

