/*
 * @Author: zhangshu
 * @email: shiliang.zhang@tuya.com
 * @LastEditors: Please set LastEditors
 * @file name: app_common.h
 * @Description: 
 * @Copyright: HANGZHOU TUYA INFORMATION TECHNOLOGY CO.,LTD
 * @Company: http://www.tuya.com
 * @Date: 2022-02-08
 * @LastEditTime: 2022-02-08
 */
#ifndef __APP_COMMON_H__
#define __APP_COMMON_H__

#ifdef __cplusplus
extern "C"
{
#endif

#include "tuya_cloud_types.h"
#include "tal_log.h"

#define MAIN_CALLBACK_DEBUG   1

#if MAIN_CALLBACK_DEBUG
#include "tal_uart.h"
#include "tlPrintf.h"
#define tal_main_debug(...)     tkl_dbg_printf(__VA_ARGS__)
#else
#define tal_main_debug(...)
#endif

#ifndef OPRT_OK
#define OPRT_OK 0
#endif
#ifndef OPRT_COM_ERROR
#define OPRT_COM_ERROR 1
#endif
#ifndef OPRT_INVALID_PARM
#define OPRT_INVALID_PARM 2
#endif

#define COMMON_LINK_SCENE                         1

#define DEFAULT_SUPPORT_BAUDRATE_9600             9600
#define DEFAULT_SUPPORT_BAUDRATE_115200           115200

#define MAX_USER_SCENE_ITEM                       32
#define DEFAULT_RF_POWER                    10
#define MIN_RF_POWER                        3
#define MAX_RF_POWER                        19

#define HIGH_LOW_TO_INT(high, low)  (((UINT16_T)(((UINT16_T)(high)) << 8)) + ((UINT16_T)((low)&0xFF)))

typedef enum{
    SEND_TYPE_UNICAST    = 0,
    SEND_TYPE_BROADCAT   = 1,
}AIR_SEND_TYPE_E;

typedef enum {
    ZCL_ID_DP_RESPONSE = 100,
    ZCL_ID_DP_REPORT,
    ZCL_ID_MCU_VERSION_REPORT,
    ZCL_ID_MCU_OTA_DATA_REQUEST,
    ZCL_ID_MCU_OTA_END,
    ZCL_ID_TTME_REQUEST,
    ZCL_ID_GATEWAY_NETWORK_STATUS_REQUEST,
    ZCL_ID_SCENE_ADD_RESPONSE,
    ZCL_ID_RECALL_SCENE,
    ZCL_ID_GROUP_BROADCAST_STANDARD,
    ZCL_ID_GROUP_BROADCAST_PRIVATE,
}ZCL_ID_E;

typedef struct{
    UINT8_T                  valid;                  //module config info valid or not
    UINT16_T                 heartbeat_period_s;     //heatbeat period in s, 10s--5*3600s, 0xffff:current period, 0xfffe:default period
    UINT16_T                 join_timeout_s;         //join timeout in s, 30s--600s, 0xffff:current period, 0xfffe:default period(180s)
    UINT16_T                 rejoin_interval_s;      //rejoin interval in s, 3s--3600s, 0xffff:current period, 0xfffe:default period(180s)
    UINT16_T                 poll_interval_ms;       //poll interval in ms, 200ms--10000ms, 0:close poll,  0xffff:current period, 0xfffe:default period(5000ms)
    UINT16_T                 poweron_poll_period_s;  //power on poll duration period in s, 10s--3000s, 0xffff:current period, 0xfffe:default period(30s)
    UINT8_T                  poll_fail_max_times;    //poll fail times(will trigger rejoin when times reach), 3-40, 0xff:current period, 0xfe:default period
    UINT8_T                  trigger_rejoin_when_send_data;   //whether trigger rejoin when send data, 0: do not trigger, 1: trigger,  0xff:current period, 0xfe:default period(1)
    UINT8_T                  rejoin_max_times;       //rejoin max times, 1--10, 0xff:current period, 0xfe:default period(1)
    UINT8_T                  tx_power;               //tx power, 3dB--19dB, 0xff:current period, 0xfe:default period(11dB)
    BOOL_T                   close_poll;             //close poll or not
}MODULE_CONFIG_INFO_T;

typedef struct
{
    UINT8_T valid;
    UINT8_T key_id;
    UINT8_T scene_id;
    UINT8_T group_id;
} USER_SCENE_INFO_T;

typedef struct {
    UINT8_T  valid;
    UINT8_T  channel;
    UINT16_T nwk_addr;
    UINT16_T panid;
} USER_NWK_INFO_T;

/**
 * @description: check sum
 * @param[in] data: checksum data
 * @param[in] len: data len
 * @return: checksum result
 */
UINT8_T make_checksum(UINT8_T *data, UINT16_T len);

/**
 * @description: my atoi
 * @param[in] s[]: sorce char
 * @return: none
 */
INT_T my_atoi(CHAR_T s[]);

/**
* @description: get user application data send sequence numbers
* @param {type} none
* @return: sequence number
*/
UINT16_T app_seq_num_get(VOID_T);

#ifdef __cplusplus
  extern "C"
  {
#endif

#endif

