/**
* @Author: zhangshu
* @email: shiliang.zhang@tuya.com
* @LastEditors: zhangshu
* @file name: app_user_flash.c
* @Description: app user flash
* @Copyright: HANGZHOU TUYA INFORMATION TECHNOLOGY CO.,LTD
* @Company: http://www.tuya.com
* @Date: 2022-11-30
* @LastEditTime: 2022-11-30
*/
#ifndef __APP_USER_FLASH_H_
#define __APP_USER_FLASH_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "tuya_cloud_types.h"
#include "app_common.h"

/**
 * @description: check if user flash valid or not
 * @param[in] none
 * @return: TRUE if user flash in valid or FALSE
*/
BOOL_T is_app_user_flash_valid(VOID_T);

/**
 * @description: read serial uart baudrate
 * @param[in] none
 * @return: baudrate
 */
UINT32_T app_user_flash_baudrate_read(VOID_T);

/**
 * @description: write serial uart baudrate
 * @param {rate} baudrate
 * @return: TRUE when write success or FALSE
 */
BOOL_T app_user_flash_baudrate_write(UINT32_T rate);

#ifndef TUYA_ZG_ROUTER
/**
 * @description: read serial module config info
 * @param[in] none
 * @return: read success or not
 */
BOOL_T app_user_flash_module_config_info_read(MODULE_CONFIG_INFO_T *info);

/**
 * @description: write module config info
 * @param {rate} baudrate
 * @return: TRUE when write success or FALSE
 */
BOOL_T app_user_flash_module_config_info_write(MODULE_CONFIG_INFO_T info);

/**
 * @description: read wakeup time
 * @param[in] none
 * @return: time
 */
UINT16_T app_user_flash_wakeup_time_read(VOID_T);

/**
 * @description: write wake up time
 * @param {time} wakeup time
 * @return: TRUE when write success or FALSE
 */
BOOL_T app_user_flash_wakeup_time_write(UINT16_T time);
#else
/**
 * @description: read RF power
 * @param[in] none
 * @return: active byte
 */
UINT8_T app_user_flash_rf_power_read(VOID_T);

/**
 * @description: write user flash active
 * @param {active} active byte
 * @return: TRUE when write success or FALSE
 */
BOOL_T app_user_flash_rf_power_write(UINT8_T power);
#endif

#if COMMON_LINK_SCENE
/**
 * @description: read scene data
 * @param[in] none
 * @return: read success or not
 */
BOOL_T app_user_scene_data_read(UINT8_T *data);

/**
 * @description: scene data write
 * @param {data} scene data
 * @return: TRUE when write success or FALSE
 */
BOOL_T app_user_scene_data_write(UINT8_T *data);

/**
 * @description: read network info
 * @param {data} scene data
 * @return: read success or not
 */
BOOL_T app_user_flash_network_info_read(UINT8_T *data);

/**
 * @description: network info data write
 * @param {data} scene data
 * @return: TRUE when write success or FALSE
 */
BOOL_T app_user_flash_network_info_write(UINT8_T *data);
#endif

#ifdef __cplusplus
}
#endif

#endif

