/**
 * @Author: Leon
 * @email: zhangpeng@tuya.com
 * @LastEditors: Leon
 * @file name: serial_protocol.h
 * @Description: serial protocol used by TuYa ZigBee modules and MCU communicated by UART
 * @Copyright: HANGZHOU TUYA INFORMATION TECHNOLOGY CO.,LTD
 * @Company: http://www.tuya.com
 * @Date: 2019-03-25 21:41:05
 * @LastEditTime: 2019-03-25 23:04:11
 */

#ifndef __UART_DATA_SEND_H
#define __UART_DATA_SEND_H


#ifdef __cplusplus
	extern "C" {
#endif

#include "tuya_cloud_types.h"

BOOL_T uart_mcu_reset_to_factory_send(VOID_T);
BOOL_T uart_product_info_req_cmd_send(VOID_T);
BOOL_T uart_zigbee_nwk_state_send(UINT8_T nwk_status);
BOOL_T uart_zigbee_cfg_rsp_send(UINT16_T seq_num);
BOOL_T uart_ack_gateway_rsp_send(UINT8_T state, UINT16_T seq_num);
BOOL_T uart_mcu_data_report_send(UINT16_T seq, BOOL_T res);
BOOL_T uart_function_test_result_send(UINT8_T percentageRssi);

BOOL_T uart_gateway_check_mcu_version_send(VOID_T);
BOOL_T uart_gateway_mcu_ota_notify_cmd_send(UINT8_T* data, UINT8_T data_len);
BOOL_T uart_gateway_response_mcu_ota_data_request_send(UINT8_T* data, UINT8_T data_len);
BOOL_T uart_mcu_ota_end_rsp_send(UINT16_T seqnum);

BOOL_T uart_mcu_check_network_rsp_send(UINT8_T state, UINT16_T seqnum);
BOOL_T uart_time_get_cmd_response_send(UINT8_T* data, UINT8_T data_len);
BOOL_T uart_gateway_network_st_req_cmd_response_send(UINT8_T* data, UINT8_T data_len);
BOOL_T uart_set_zigbee_nwk_info_rsp_send(UINT16_T seqnum);
BOOL_T uart_mcu_data_broadcast_send_rsp_send(UINT16_T seqnum);
BOOL_T uart_gateway_read_dp_value_send(UINT8_T* data, UINT8_T data_len, BOOL_T with_seq);
BOOL_T uart_beacon_notify_send(VOID_T);

BOOL_T uart_air_zigbee_dp_req_send(UINT8_T cmd, UINT8_T *data, UINT8_T data_len, UINT16_T seqnum);

#ifndef TUYA_ZG_ROUTER
BOOL_T uart_set_wake_up_time_rsp_send(UINT16_T seqnum, BOOL_T response);
#endif

#if COMMON_LINK_SCENE
/**
 * serial mcu report switch id response send
 * @param[in]{seqnum} seq number
 * @param[in]{response} response result
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_mcu_report_switch_id_rsponse_send(UINT16_T seqnum, BOOL_T response);

/**
 * serial mcu report switch id response send
 * @param[in]{kid} key id
 * @param[in]{gid} group id
 * @param[in]{sid} scene id
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_multicast_set_cmd_send(UINT8_T kid, UINT16_T gid, UINT8_T sid);

/**
 * serial uart multicast response send
 * @param[in]{kid} key id
 * @param[in]{gid} group id
 * @param[in]{sid} scene id
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_multicast_response_send(UINT16_T seqnum, UINT8_T cmd, UINT8_T result);
#endif

BOOL_T push_uart_data(UINT8_T *data, UINT8_T len, UINT16_T seq, BOOL_T is_resend);
BOOL_T pop_uart_queue(UINT8_T *data, UINT8_T *data_len);

VOID_T uart_queue_init(VOID_T);

#ifdef __cplusplus
}
#endif

#endif /*__SERIAL_PROTOCOL_H*/

