/**
* @Author: zhangshu
* @email: shiliang.zhang@tuya.com
* @LastEditors: zhangshu
* @file name: air_data_received_handle.c
* @Description: air data received handle
* @Copyright: HANGZHOU TUYA INFORMATION TECHNOLOGY CO.,LTD
* @Company: http://www.tuya.com
* @Date: 2022-11-13
* @LastEditTime: 2022-11-13
*/
#include "tal_sw_timer.h"
#include "tal_memory.h"

#include "air_data_send.h"
#include "air_data_received_handle.h"

#include "uart_data_send.h"
#include "uart_data_received_handle.h"

#include "app_user_flash.h"
#ifdef TUYA_ZG_ROUTER
#include "dp_data_backup.h"
#endif

#define AIR_DATA_RECEIVED_HANDLE_DBG 0

#if AIR_DATA_RECEIVED_HANDLE_DBG
#define air_data_recived_handle_debug(...)      tal_main_debug(__VA_ARGS__)
#else
#define air_data_recived_handle_debug(...)
#endif

//air data receivew event
TIMER_ID notify_mcu_report_dp_fiter_event_id;
TIMER_ID broadcast_delay_event_id;

STATIC VOID_T notify_mcu_report_dp_fiter_event_handler(TIMER_ID timer_id, VOID_T *arg);
STATIC VOID_T broadcast_delay_event_handler(TIMER_ID timer_id, VOID_T *arg);

STATIC BOOL_T g_recevied_dp_report_notify = FALSE;
STATIC AIR_SEND_TYPE_E g_air_message_type  = SEND_TYPE_UNICAST;
STATIC UINT8_T g_group_ability = 0;

STATIC VOID_T save_scene_info(dp_data_t* dp_inf);
STATIC BOOL_T  dp_scene_info_check_and_handle(UINT8_T* buffer, UINT8_T buffer_len);
STATIC BOOL_T air_dp_cmd_handle(UINT8_T* data, UINT8_T data_len, ZG_DELIVERY_MODE_E message_type);

#if COMMON_LINK_SCENE
STATIC USER_SCENE_INFO_T g_scene_info[MAX_USER_SCENE_ITEM];
STATIC VOID_T app_user_scene_info_load(VOID_T);
#endif

/**
 * @brief notify mcu report dp event handler
 *
 * @param[in] timer_id: event timer id
 * @param[in] arg: event data
 * @return NULL
 */
STATIC VOID_T notify_mcu_report_dp_fiter_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
    g_recevied_dp_report_notify = FALSE;
}

/**
 * @brief broadcast delay set event handler
 *
 * @param[in] timer_id: event timer id
 * @param[in] arg: event data
 * @return NULL
 */
STATIC VOID_T broadcast_delay_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
    g_air_message_type = SEND_TYPE_UNICAST;
}

#if COMMON_LINK_SCENE
/**
 * @brief save scene info
 *
 * @param[in] dp_inf: dp info
 * @return NULL
 */
STATIC VOID_T save_scene_info(dp_data_t* dp_inf)
{
  if(NULL == dp_inf){
    return;
  }

  if(0x11 == dp_inf->dp_len){
    if((dp_inf->dp_data[3] > MAX_USER_SCENE_ITEM) || (dp_inf->dp_data[3] == 0)){
      return;
    }

    g_scene_info[dp_inf->dp_data[3] -1].key_id = dp_inf->dp_data[3];
    g_scene_info[dp_inf->dp_data[3] -1].group_id = dp_inf->dp_data[7];
    g_scene_info[dp_inf->dp_data[3] -1].scene_id =  dp_inf->dp_data[11];
    if(dp_inf->dp_data[11] == 0x00){
       g_scene_info[dp_inf->dp_data[3] -1].valid = 0x00;
    }
    else{
       g_scene_info[dp_inf->dp_data[3] -1].valid = 0xaa;
       uart_multicast_set_cmd_send(g_scene_info[dp_inf->dp_data[3] -1].key_id, g_scene_info[dp_inf->dp_data[3] -1].group_id, g_scene_info[dp_inf->dp_data[3] -1].scene_id);
    }
    UINT8_T buff[MAX_USER_SCENE_ITEM*SIZEOF(USER_SCENE_INFO_T)];
    tal_system_memset(&buff[0], 0, MAX_USER_SCENE_ITEM*SIZEOF(USER_SCENE_INFO_T));
    tal_system_memcpy(&buff[0], g_scene_info, MAX_USER_SCENE_ITEM*SIZEOF(USER_SCENE_INFO_T));
    for(UINT8_T i=0; i <MAX_USER_SCENE_ITEM; i++) {
        if (g_scene_info[i].valid) {
            air_data_recived_handle_debug("valid keyid %d, sceneid %x, groupid %x\r\n", 
                g_scene_info[i].key_id, g_scene_info[i].scene_id, g_scene_info[i].group_id);
        }
    }
    app_user_scene_data_write(&buff[0]);
  }
}

/**
 * @brief save scene info
 *
 * @param[in] buffer: buff data
 * @param[in] buffer_len: buff len
 * @return scene info valid or not
 */
STATIC BOOL_T  dp_scene_info_check_and_handle(UINT8_T* buffer, UINT8_T buffer_len)
{
    UINT8_T i = 0;
    dp_data_t dp_inf;
    UINT16_T seq = 0;
    UINT8_T *pTr = NULL;
    
    if ((NULL == buffer) || (buffer_len < 2))
    {
        air_data_recived_handle_debug("buffer error \r\n");
        return FALSE;
    }
    pTr = &buffer[2];
    seq = (UINT16_T)(buffer[0] << 8) + (UINT16_T)(buffer[1]);
    
    for (i = 0; i < buffer_len - 2;) {
        tal_system_memset(&dp_inf, 0, sizeof(dp_data_t));
        
        dp_inf.dpid = pTr[i];
        dp_inf.dp_type = (DP_TYPE_E)pTr[i + 1];
        dp_inf.dp_len = pTr[ i + 2] << 8;
        dp_inf.dp_len |= pTr[ i + 3];
        air_data_recived_handle_debug("dp info id %x, type %x, len %d\r\n", dp_inf.dpid, dp_inf.dp_type, dp_inf.dp_len);
        if ((dp_inf.dp_len < 60) && (dp_inf.dp_len > 0)) {
            tal_system_memcpy(dp_inf.dp_data, &pTr[i+4], dp_inf.dp_len);
        }
        else {
            return FALSE;
        }

        if (17 == dp_inf.dpid) {
            save_scene_info(&dp_inf);
            air_scene_add_response_send(seq);
            return TRUE;
        }
        
        i += (dp_inf.dp_len + 4);
        if (buffer_len  - i < 7) {
            break;
        }
    }
    return FALSE;
}

/**
* @description: load user scene info
* @return: none
*/
STATIC VOID_T app_user_scene_info_load(VOID_T)
{
    UINT8_T buffer[MAX_USER_SCENE_ITEM*SIZEOF(USER_SCENE_INFO_T)];
    tal_system_memset(g_scene_info, 0, MAX_USER_SCENE_ITEM*SIZEOF(USER_SCENE_INFO_T));
    tal_system_memset(&buffer[0], 0, MAX_USER_SCENE_ITEM*SIZEOF(USER_SCENE_INFO_T));

    app_user_scene_data_read(&buffer[0]);
    tal_system_memcpy(g_scene_info, &buffer[0], MAX_USER_SCENE_ITEM*SIZEOF(USER_SCENE_INFO_T));
    for(UINT8_T i=0; i <MAX_USER_SCENE_ITEM; i++) {
        if (0xAA == g_scene_info[i].valid) {
            air_data_recived_handle_debug("load valid keyid %d, sceneid %x, groupid %x\r\n", 
                g_scene_info[i].key_id, g_scene_info[i].scene_id, g_scene_info[i].group_id);
        }
    }
}

/**
* @description: clear user scene info
* @param[in] group_id: group id
* @return: none
*/
VOID_T app_user_scene_info_clear(UINT16_T group_id)
{
    UINT8_T buff[MAX_USER_SCENE_ITEM*SIZEOF(USER_SCENE_INFO_T)];
    UINT8_T gid = (UINT8_T)group_id;

    air_data_recived_handle_debug("scene info clear gid %x\r\n", group_id);
    if (0xFFFF == group_id) {
        tal_system_memset(g_scene_info, 0, MAX_USER_SCENE_ITEM*SIZEOF(USER_SCENE_INFO_T));
        tal_system_memset(&buff[0], 0, MAX_USER_SCENE_ITEM*SIZEOF(USER_SCENE_INFO_T));
        app_user_scene_data_write(&buff[0]);
        return;
    }

    for(UINT8_T i=0; i <MAX_USER_SCENE_ITEM; i++) {
        if (0xAA == g_scene_info[i].valid) {
            if (gid == g_scene_info[i].group_id) {
                g_scene_info[i].valid = 0x00;
                g_scene_info[i].key_id = 0;
                g_scene_info[i].group_id = 0;
                g_scene_info[i].scene_id = 0;
            }
        }
    }
    tal_system_memset(&buff[0], 0, MAX_USER_SCENE_ITEM*SIZEOF(USER_SCENE_INFO_T));
    tal_system_memcpy(&buff[0], g_scene_info, MAX_USER_SCENE_ITEM*SIZEOF(USER_SCENE_INFO_T));
    app_user_scene_data_write(&buff[0]);
}

/**
* @description: load user scene info
* @param[in] switch_id: switch id
* @param[in] *group_id: group id
* @param[in] *scene_id: scene id
* @return: return TRUE when valid, else return FALSE
*/
BOOL_T app_switch_id_valid_check_and_get(UINT8_T switch_id, UINT8_T *group_id, UINT8_T *scene_id)
{
    if(g_scene_info[switch_id-1].valid == 0xaa){
        *group_id = g_scene_info[switch_id-1].group_id;
        *scene_id = g_scene_info[switch_id-1].scene_id;
        return TRUE;
    }
    return FALSE;
}
#endif

/**
* @description: air dp commmand handler
* @param {data} air data
* @param {data_len} air data len
* @param {message_type} zigbee delivery mode
* @return: none
*/
STATIC BOOL_T air_dp_cmd_handle(UINT8_T* data, UINT8_T data_len, ZG_DELIVERY_MODE_E message_type)
{
    UINT16_T seqNum = 0;
    UINT8_T serial_data_len = 0;
    UINT8_T cmd = 0;
    UINT8_T* ptr= NULL;

#if COMMON_LINK_SCENE
    if (dp_scene_info_check_and_handle(data, data_len)) {
        return TRUE;
    }
#endif

    if((ZG_UNICAST_MODE != message_type)&&(ZG_ADDR_UNKNOWN != message_type)&&(ZG_BINDING_MODE != message_type)){
        if(!g_group_ability){ // if device only support unicast ability
            air_data_recived_handle_debug("message send type broadcast\r\n");
            g_air_message_type = SEND_TYPE_BROADCAT;
            tal_sw_timer_start(broadcast_delay_event_id, 1000, TAL_TIMER_ONCE);
        }
    }
    else{
        air_data_recived_handle_debug("message send type unicast\r\n");
        g_air_message_type = SEND_TYPE_UNICAST;
    }

    cmd = ZIGBEE_DATA_REQ_CMD;

    if((ZG_UNICAST_MODE != message_type)&&(ZG_ADDR_UNKNOWN != message_type)&&(ZG_BINDING_MODE != message_type)&&(g_group_ability)){ //if device have group ability
      cmd = ZIGBEE_DATA_REQ_GROUP_CMD; // receive group or broadcast muliticast command, use new command 0x2A
    }

    /*payload first and second byte is sequence number in version 2*/
    ptr = data;/*the start three byte is frame control, sequence and command id*/
    seqNum = *ptr << 8 ;
    ptr++;
    seqNum |= *ptr;
    ptr++;
    air_data_recived_handle_debug("air seq=%x\n",seqNum);
    serial_data_len = data_len-2;
#ifdef TUYA_ZG_ROUTER
    air_data_recived_handle_debug("add air rceived dp data to filter buf\n");
    dev_data_req_handle(ptr, serial_data_len);  // add air rceived dp data
#endif

    uart_air_zigbee_dp_req_send(cmd, ptr, serial_data_len, seqNum);
    return TRUE;
}

/**
* @description: get air send message type
* @param  none
* @return: air send message type, unicast or broadcast
*/
AIR_SEND_TYPE_E air_message_type_get(VOID_T)
{
    return g_air_message_type;
}

/**
* @description: group ability set
* @param {type} group ability set
* @return: none
*/
VOID_T group_ability_set(UINT8_T type)
{
  g_group_ability = type;
}

/**
 * Af_command_parse_handle
 * Cluster: serial protocol private cluster command parse callback
 * @param {cmd} air command of private cluster
 * @param {mode} delivery mode
 * @param {buff} air data 
 * @param {buff_len} air data len
 * @return operate status
 */
OPERATE_RET air_private_cluster_command_parse_handle(UINT8_T cmd, ZG_DELIVERY_MODE_E mode, UINT8_T *buff, UINT8_T buff_len)
{
    BOOL_T     status = TRUE;

    switch (cmd){
        case SERIAL_PROTOCOL_DATA_REQ:
            air_data_recived_handle_debug("air receive air dp cmd\r\n");
            status = air_dp_cmd_handle(buff, buff_len, mode);
        break;

        case SERIAL_PROTOCOL_DATA_QUERY:
        air_data_recived_handle_debug("air receive data query\r\n");
        if(g_recevied_dp_report_notify == FALSE){ // filter duplicate data
            g_recevied_dp_report_notify = TRUE;
            tal_sw_timer_start(notify_mcu_report_dp_fiter_event_id, 5000, TAL_TIMER_ONCE);
            uart_gateway_read_dp_value_send(buff, buff_len, TRUE);
        }
        break;

        case QUERY_SPECIFIC_DP_CMD: 
            air_data_recived_handle_debug("air receive query specific dp cmd\r\n");
            uart_gateway_read_dp_value_send(buff, buff_len, TRUE);
        break;

        case SERIAL_PROTOCOL_TIME_REQ:
            air_data_recived_handle_debug("air receive get time response\r\n");
            uart_time_get_cmd_response_send(buff, buff_len);
        break;

        /*1.0.9 new add 0x25 mcu check network st*/
        case SERIAL_PROTOCOL_GW_NETWORK_ST_REQ:
            air_data_recived_handle_debug("air receivew gateway network status request\r\n");
            uart_gateway_network_st_req_cmd_response_send(buff, buff_len);
            stop_gw_st_check_timeout_event();
        break;

        case SERIAL_PROTOCOL_MCU_OTA_VER_REQ:
            air_data_recived_handle_debug("air receivew ota version check\r\n");
            uart_gateway_check_mcu_version_send();
        break;

        case SERIAL_PROTOCOL_MCU_OTA_NOTIFY:
            air_data_recived_handle_debug("air receivew ota notify\r\n");
            uart_gateway_mcu_ota_notify_cmd_send(buff, buff_len);
        break;

        case SERIAL_PROTOCOL_MCU_OTA_DATA_RSP:
            air_data_recived_handle_debug("air receivew mcu OTA data response\r\n");
            uart_gateway_response_mcu_ota_data_request_send(buff, buff_len);
        break;

        default:
            air_data_recived_handle_debug("unsupported air cmd!!!\r\n");
            status = FALSE;
        break;
    }

    if(TRUE == status){
        return OPRT_OK;
    }else{
        return OPRT_COM_ERROR;
    }
}

/**
 * @brief air data reveiced handle event init
 *
 * @param[in] none
 * @return none
 */
VOID_T air_data_received_handle_event_init(VOID_T)
{
#if COMMON_LINK_SCENE
    app_user_scene_info_load();
#endif
    tal_sw_timer_create(notify_mcu_report_dp_fiter_event_handler, NULL, &notify_mcu_report_dp_fiter_event_id);
    tal_sw_timer_create(broadcast_delay_event_handler, NULL, &broadcast_delay_event_id);
}

