/*
 * @Author: zhangshu
 * @email: shiliang.zhang@tuya.com
 * @LastEditors: Please set LastEditors
 * @file name: app_dev_register.c
 * @Description: 
 * @Copyright: HANGZHOU TUYA INFORMATION TECHNOLOGY CO.,LTD
 * @Company: http://www.tuya.com
 * @Date: 2022-11-30
 * @LastEditTime: 2022-11-30
 */
#include "tal_memory.h"

#include "app_dev_register.h"
#include "tal_network_mgr.h"
#include "tal_endpoint_register.h"

#ifndef GET_ARRAY_LEN
#define GET_ARRAY_LEN(x)    (SIZEOF(x) / SIZEOF(x[0]))
#endif

#define GROUP_ATTR_LIST                                                                                                      \
    {0x0000, ATTR_BITMAP8_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x00},      /* 12 / Groups / name support*/ \
    {0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x0002}, /* 13 / Groups / cluster revision*/

#define SCENE_ATTR_LIST                                                                                                       \
    {0x0000, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x00},        /* 15 / Scenes / scene count*/   \
    {0x0001, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x00},    /* 16 / Scenes / current scene*/ \
    {0x0002, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x0000}, /* 16 / Scenes / current group*/ \
    {0x0003, ATTR_BOOLEAN_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x00},  /* 17 / Scenes / scene valid*/   \
    {0x0004, ATTR_BITMAP8_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x00},  /* 18 / Scenes / name support*/  \
    {0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x0002}, /* 19 / Scenes / cluster revision*/

#if COMMON_LINK_SCENE
#define PRIVATE_SCENE_ATTR_LIST \
    {0x0000, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_TOKEN_FAST|ATTR_MASK_SINGLETON), 0, (UINT8_T*)0x00 }, /* current positiong lift percentage*/\
    { 0x0001, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE), 0, (UINT8_T*)0x00 }, /* 16 / Scenes / current scene*/\
    { 0x0002, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T*)0x0000 }, /* 16 / Scenes / current group*/\
    { 0x0003, ATTR_BOOLEAN_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE), 0, (UINT8_T*)0x00 }, /* 17 / Scenes / scene valid*/\
    { 0x0004, ATTR_BITMAP8_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE), 0, (UINT8_T*)0x00 }, /* 18 / Scenes / name support*/\
    {0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x0002},
#else
#define PRIVATE_ATTR_LIST \
    {0x0000, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_TOKEN_FAST|ATTR_MASK_SINGLETON), 0, (UINT8_T*)0x00 }, /* current positiong lift percentage*/\
    {0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x0002},
#endif

const TAL_ATTR_T group_attr_list[] = {
    GROUP_ATTR_LIST};

const TAL_ATTR_T scene_attr_list[] = {
    SCENE_ATTR_LIST};

#if COMMON_LINK_SCENE
const TAL_ATTR_T g_private_attr_list[] = {
    PRIVATE_SCENE_ATTR_LIST};
#else
const TAL_ATTR_T g_private_attr_list[] = {
    PRIVATE_ATTR_LIST};
#endif

#define DEF_CLUSTER_GROUPS_CLUSTER_ID(a) \
    {CLUSTER_GROUPS_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a))},

#define DEF_CLUSTER_SCENES_CLUSTER_ID(a) \
    {CLUSTER_SCENES_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a))},

#define DEF_CLUSTER_PRIVATE_CLUSTER_ID(a) \
    { CLUSTER_PRIVATE_TUYA_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a)) },


// server cluster list
CONST TAL_CLUSTER_T app_server_cluster_list[] = {
    DEF_CLUSTER_GROUPS_CLUSTER_ID(group_attr_list)
    DEF_CLUSTER_SCENES_CLUSTER_ID(scene_attr_list)
    DEF_CLUSTER_PRIVATE_CLUSTER_ID(g_private_attr_list)
};

// client cluster list
//CONST TAL_CLUSTER_T app_client_cluster_list[] = {
    //DEF_CLUSTER_IDENTIFY_CLUSTER_ID(identify_client_attr_list)
    //DEF_CLUSTER_GROUPS_CLUSTER_ID(group_client_attr_list)
    //DEF_CLUSTER_ON_OFF_CLUSTER_ID(onoff_client_attr_list)
    //DEF_CLUSTER_ZLL_COMMISSIONING_CLUSTER_ID(zll_client_attr_list)
//};

#define SERVER_CLUSTER_NUM GET_ARRAY_LEN(app_server_cluster_list)
#define CLIENT_CLUSTER_NUM GET_ARRAY_LEN(app_client_cluster_list)

//endpoint descriptor
TAL_ENDPOINT_T dev_endpoint_desc[] = {
    {1, ZHA_PROFILE_ID, ZG_DEVICE_ID_SMART_PLUG, SERVER_CLUSTER_NUM, (TAL_CLUSTER_T *)&app_server_cluster_list[0], 0, NULL},
};

STATIC VOID_T app_zg_node_init(VOID_T);
STATIC VOID_T app_join_config_init(VOID_T);

#ifndef TUYA_ZG_ROUTER
MODULE_CONFIG_INFO_T g_module_config_info;

/**
* @description: sleep end device config load
* @param  none
* @return: config init success or not
*/
BOOL_T app_sleep_end_device_config_load(VOID_T)
{
    BOOL_T res = FALSE;

    //wakeup time init
    UINT16_T wakeup_time = 0;
    wakeup_time = app_user_flash_wakeup_time_read();
    if ((wakeup_time >= MIN_WAKE_UP_TIME)&&(wakeup_time <= MAX_WAKE_UP_TIME)) {
        wake_up_time_set(wakeup_time);
    }else {
        wake_up_time_set(DEFAULT_WAKE_UP_TIME);
    }

    //module config info init
    tal_system_memset(&g_module_config_info, 0, sizeof(MODULE_CONFIG_INFO_T));
    app_user_flash_module_config_info_read(&g_module_config_info);
    if (CHECK_RES_VALID == g_module_config_info.valid) {
        if (g_modle_network_info.heartbeat_period_s > MAX_HEART_PERIOD){
            g_modle_network_info.heartbeat_period_s = DEFAULT_HEART_PERIOD;
        }
        else if (g_modle_network_info.heartbeat_period_s < MIN_HEART_PERIOD){
            g_modle_network_info.heartbeat_period_s = MIN_HEART_PERIOD;
        }

        if (g_modle_network_info.join_timeout_s > MAX_JION_PERIOD){
            g_modle_network_info.join_timeout_s = DEFAULT_JION_PERIOD;
        }
        else if (g_modle_network_info.join_timeout_s < MIN_JION_PERIOD){
            g_modle_network_info.join_timeout_s = MIN_JION_PERIOD;
        }

        if (g_modle_network_info.rejoin_interval_s > MAX_REJION_PERIOD){
            g_modle_network_info.rejoin_interval_s = DEFAULT_REJION_PERIOD;
        }
        else if (g_modle_network_info.rejoin_interval_s < MIN_REJION_PERIOD){
            g_modle_network_info.rejoin_interval_s = MIN_REJION_PERIOD;
        }

        if (g_modle_network_info.poll_interval_ms > MAX_POLL_PERIOD){
            g_modle_network_info.poll_interval_ms = DEFAULT_POLL_PERIOD;
        }
        else if (g_modle_network_info.poll_interval_ms < MIN_POLL_PERIOD){
            g_modle_network_info.poll_interval_ms = MIN_POLL_PERIOD;
        }


        if (g_modle_network_info.poweron_poll_period_s > MAX_POWER_ON_POLL_PERIOD){
            g_modle_network_info.poweron_poll_period_s = DEFAULT_POWER_ON_POLL_PERIOD;
        }
        else if (g_modle_network_info.poweron_poll_period_s < MIN_POWER_ON_POLL_PERIOD){
            g_modle_network_info.poweron_poll_period_s = MIN_POWER_ON_POLL_PERIOD;
        }

        if (g_modle_network_info.poll_fail_max_times > MAX_POLL_FAIL_TIMES){
            g_modle_network_info.poll_fail_max_times = DEFAULT_POLL_FAIL_TIMES;
        }
        else if (g_modle_network_info.poll_fail_max_times < MIN_POLL_FAIL_TIMES){
            g_modle_network_info.poll_fail_max_times = MIN_POLL_FAIL_TIMES;
        }


        if (g_modle_network_info.trigger_rejoin_when_send_data > 1){
            g_modle_network_info.trigger_rejoin_when_send_data = 1;
        }


        if (g_modle_network_info.rejoin_max_times > MAX_REJION_TRY_TIMES){
            g_modle_network_info.rejoin_max_times = DEFAULT_REJION_TRY_TIMES;
        }
        else if (g_modle_network_info.rejoin_max_times < MIN_REJION_TRY_TIMES){
            g_modle_network_info.rejoin_max_times = MIN_REJION_TRY_TIMES;
        }


        if (g_modle_network_info.tx_power > MAX_RF_POWER){
            g_modle_network_info.tx_power = DEFAULT_RF_POWER;
        }
        else if (g_modle_network_info.tx_power < MIN_RF_POWER){
            g_modle_network_info.tx_power = MIN_RF_POWER;
        }

        if (g_modle_network_info.close_poll > 1){
            g_modle_network_info.close_poll = FALSE;
        }
        res = TRUE;
    }
    else {
        g_modle_network_info.heartbeat_period_s = DEFAULT_HEART_PERIOD;
        g_modle_network_info.join_timeout_s =   DEFAULT_JION_PERIOD;
        g_modle_network_info.rejoin_interval_s = DEFAULT_REJION_PERIOD;
        g_modle_network_info.poll_interval_ms = DEFAULT_POLL_PERIOD;
        g_modle_network_info.poweron_poll_period_s = DEFAULT_POWER_ON_POLL_PERIOD;
        g_modle_network_info.poll_fail_max_times = DEFAULT_POLL_FAIL_TIMES;
        g_modle_network_info.trigger_rejoin_when_send_data = 1;
        g_modle_network_info.rejoin_max_times = DEFAULT_REJION_TRY_TIMES;
        g_modle_network_info.tx_power = DEFAULT_RF_POWER;
        g_modle_network_info.close_poll = FALSE;
        res = FALSE;
    }
}
#endif

/**
 * @brief register router or sleep device
 * @return none
 */
STATIC VOID_T app_zg_node_init(VOID_T)
{
    TAL_ZG_NODE_CFG_T node_config;

    tal_system_memset(&node_config, 0, SIZEOF(TAL_ZG_NODE_CFG_T));

#ifdef TUYA_ZG_ROUTER
    //basic config of zg router
    node_config.node_type = ZG_ROUTER;//zigbee type, router, end device or sleep end device
    node_config.tx_power = 10;//set tx power
    node_config.scan_interval = 0;//beacon scan interval
    node_config.scan_duration = ZG_SCAN_DURATION_3;//beacon scan interval 153.6 ms
    //router config
    node_config.config.router_cfg.reserved = 0;
#else
    //basic config of zg sleepy end device
    node_config.node_type = ZG_SLEEPY_END_DEVICE,
    node_config.tx_power = 10,
    node_config.scan_interval = 300,
    node_config.scan_duration = ZG_SCAN_DURATION_3,
    //sleep end device poll config
    node_config.config.sleep_ed_cfg.poll_config.forever_flag = TRUE,
    node_config.config.sleep_ed_cfg.poll_config.fast_swicth_parent = 0,
    node_config.config.sleep_ed_cfg.poll_config.interval_ms = (UINT16_T)g_modle_network_info.poll_interval_ms,
    node_config.config.sleep_ed_cfg.poll_config.max_failed_times = g_modle_network_info.poll_fail_max_times,
    node_config.config.sleep_ed_cfg.poll_config.duration_after_data_ms = 1000,
    node_config.config.sleep_ed_cfg.poll_config.duration_after_join_ms = 60000,
    node_config.config.sleep_ed_cfg.poll_config.duration_after_rejoin_ms = ((UINT32_T)(g_modle_network_info.rejoin_interval_s)) * 1000UL,
    //sleep end device rejoin config
    node_config.config.sleep_ed_cfg.rejoin_config.power_on_active = 1,
    node_config.config.sleep_ed_cfg.rejoin_config.send_data_active = g_modle_network_info.trigger_rejoin_when_send_data,
    node_config.config.sleep_ed_cfg.rejoin_config.attempts = g_modle_network_info.rejoin_max_times,
    node_config.config.sleep_ed_cfg.rejoin_config.interval = 1000,
#endif
    //config zigbee node
    tal_zg_node_config(&node_config);
}

/**
 * @brief config join strategy
 * @return none
 */
STATIC VOID_T app_join_config_init(VOID_T)
{
    TAL_ZG_JOIN_CFG_T join_config;

    tal_system_memset(&join_config, 0, SIZEOF(TAL_ZG_JOIN_CFG_T));

    join_config.auto_join_power_on_flag = FALSE;//defalut power on start join or not
    join_config.auto_join_remote_leave_flag = FALSE;//default start join or not when receivew remote leave request
#ifdef TUYA_ZG_ROUTER
    join_config.join_timeout = 180*1000;//join timeout times
#else
join_config.join_timeout = ((UINT32_T)(g_modle_network_info.join_timeout_s)) * 1000;//join timeout times
#endif

    //config join prameter
    tal_zg_join_config(&join_config);
}

/**
 * @brief register zigbee parameter
 * @return none
 */
VOID_T app_dev_zigbee_init(VOID_T)
{
    //register zigbee endpoint
    tal_zg_endpoint_register(dev_endpoint_desc, GET_ARRAY_LEN(dev_endpoint_desc));

    //zigbee node configuration
    app_zg_node_init();

    //zigbee joining network configuration
    app_join_config_init();
}

