/**
* @Author: zhangshu
* @email: shiliang.zhang@tuya.com
* @LastEditors: zhangshu
* @file name: baudrate_adaptive.c
* @Description: baudrate adaptive
* @Copyright: HANGZHOU TUYA INFORMATION TECHNOLOGY CO.,LTD
* @Company: http://www.tuya.com
* @Date: 2022-11-13
* @LastEditTime: 2022-11-13
*/
#include "tal_sw_timer.h"
#include "tal_network_mgr.h"
#include "tal_uart.h"

#include "app_user_flash.h"
#include "uart_data_received_handle.h"
#include "baudrate_adaptive.h"
#include "uart_data_send.h"

#define BAUDRATE_ADAPTIVE_DBG 0

#if BAUDRATE_ADAPTIVE_DBG
#define baudrate_adaptive_debug(...)      tal_main_debug(__VA_ARGS__)
#else
#define baudrate_adaptive_debug(...)
#endif

//3 times in 9600 baudrate, 3 times in 115200 baudrate
#define MAX_BEACON_NOTIFY_TIMES             6

//serial common prapare event
TIMER_ID baudrate_adaptive_event_id;
#ifndef TUYA_ZG_ROUTER
TIMER_ID check_wakeup_io_event_id;
#endif

STATIC VOID_T baudrate_adaptive_event_handler(TIMER_ID timer_id, VOID_T *arg);
#ifndef TUYA_ZG_ROUTER
STATIC VOID_T check_wakeup_io_event_handler(TIMER_ID timer_id, VOID_T *arg);
#endif

STATIC BOOL_T  g_get_baudrate_sucess = FALSE;
STATIC UINT8_T  g_received_beacon_notify_resopnse = 0;
STATIC UINT32_T g_baudrate = 0;
STATIC BAUDRATE_ADAPTIVE_TYPE_E g_adaptive_type = BAUDRATE_ADAPTIVE_NONE;

#ifndef TUYA_ZG_ROUTER
STATIC VOID_T check_input_io_state(VOID_T);
#endif

extern VOID_T common_link_uart_init(UINT_T baudrate);

/**
 * @brief uart beacon notiry event handler
 *
 * @param[in] timer_id: event timer id
 * @param[in] arg: event data
 * @return NULL
 */
STATIC VOID_T baudrate_adaptive_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
    if (BAUDRATE_ADAPTIVE_TYPE_CHECK_INFO == g_adaptive_type) {
        STATIC UINT8_T baudrate_trigger = 0;
        TAL_ZG_NWK_STATUS_E  state;
        
        baudrate_adaptive_debug("check product info\n");
        
        if (NWK_STATE_OK == device_work_state_get()){
            state = tal_zg_nwk_status_get();
            if ((TAL_ZG_NWK_POWER_ON_ONLINE != state) && (TAL_ZG_NWK_JOIN_OK != state) && (TAL_ZG_NWK_REJOIN_OK != state) && \
                    (TAL_ZG_NWK_LOST != state) && ((TAL_ZG_NWK_JOIN_START != state))){
                report_zigbee_network_state_to_mcu(NWK_NOT_JOIN, FALSE);
            }
        }
        else{
            if (!g_get_baudrate_sucess){
                if (baudrate_trigger == 1){
                    baudrate_trigger = 0;
                    g_baudrate = DEFAULT_SUPPORT_BAUDRATE_115200;
                    baudrate_adaptive_debug("try 115200\r\n");
                }
                else{
                    baudrate_trigger = 1;
                    g_baudrate = DEFAULT_SUPPORT_BAUDRATE_9600;
                    baudrate_adaptive_debug("try 9600\r\n");
                }
                common_link_uart_init(g_baudrate);
            }
            uart_product_info_req_cmd_send();
            tal_sw_timer_start(baudrate_adaptive_event_id, 1000UL, TAL_TIMER_ONCE);
        }
    }
    else if (BAUDRATE_ADAPTIVE_TYPE_BEACON == g_adaptive_type) {
        STATIC UINT32_T baud = DEFAULT_SUPPORT_BAUDRATE_9600;
        STATIC UINT8_T  g_send_beacon_notify_times = 0;

        if (g_send_beacon_notify_times < MAX_BEACON_NOTIFY_TIMES){
            tal_uart_deinit(SERIAL_PROTOCOL_UART_ID);
            common_link_uart_init(baud);
            if (0 == g_received_beacon_notify_resopnse){
                tal_sw_timer_start(baudrate_adaptive_event_id, 100, TAL_TIMER_ONCE);
            }
            else{
                g_received_beacon_notify_resopnse = 0;
                g_send_beacon_notify_times = 0;
            }
            baudrate_adaptive_debug("beacon time =%d, baud %d\r\n", g_send_beacon_notify_times, baud);
            uart_beacon_notify_send();
        
            g_send_beacon_notify_times++;
            if (g_send_beacon_notify_times >= 3) {
                baud = DEFAULT_SUPPORT_BAUDRATE_115200;
            }
        }
        else{
            g_received_beacon_notify_resopnse = 0;
            g_send_beacon_notify_times = 0;
        }
    }
}

#ifndef TUYA_ZG_ROUTER
/**
 * @brief check wakeup io event handler
 *
 * @param[in] timer_id: event timer id
 * @param[in] arg: event data
 * @return NULL
 */
STATIC VOID_T check_wakeup_io_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
    STATIC UINT16_T check_io_times = 0;
    TUYA_GPIO_LEVEL_E alarm_value = TUYA_GPIO_LEVEL_HIGH;
    
    if (alarm_value == TUYA_GPIO_LEVEL_HIGH){
        baudrate_adaptive_debug("check_input_io_state check_io_times =%d\r\n",check_io_times);
        check_io_times++;
    	if(check_io_times > 480){
    		check_io_times = 0;
    		tal_error_code_set(MCU_LONG_WAKEUP_E_CODE);
    	}
        tal_sw_timer_start(check_wakeup_io_event_id, (WAKE_UP_STAY_TIME - 50), TAL_TIMER_ONCE);
    }
    else{
    	baudrate_adaptive_debug("check_input_io_state sleep\r\n",check_io_times);
    	check_io_times = 0;
    }
}
#endif

/**
 * @brief delay start baudrate adaptive
 *
 * @param[in] delay_ms: delay time in ms
 * @param[in] type: baudrate adaptive type
 * @return NULL
 */
VOID_T baudrate_adaptive_start(UINT32_T delay_ms, BAUDRATE_ADAPTIVE_TYPE_E type)
{
    if (BAUDRATE_ADAPTIVE_NONE != g_adaptive_type) {
        baudrate_adaptive_debug("alreay in baudrate adaptive!!!\r\n");
        return;
    }
    g_adaptive_type = type;
    tal_sw_timer_start(baudrate_adaptive_event_id, delay_ms, TAL_TIMER_ONCE);
}

/**
 * @brief baudrate adaptive success callback
 *
 * @param[in] NULL
 * @return NULL
 */
VOID_T baudrate_adaptive_success_callback(VOID_T)
{
    if ((BAUDRATE_ADAPTIVE_TYPE_BEACON != g_adaptive_type)&&(BAUDRATE_ADAPTIVE_TYPE_CHECK_INFO != g_adaptive_type)) {
        baudrate_adaptive_debug("adaptive error\r\n");
        return;
    }

    if ((DEFAULT_SUPPORT_BAUDRATE_9600 == g_baudrate)||(DEFAULT_SUPPORT_BAUDRATE_115200 == g_baudrate)) {
        baudrate_adaptive_debug("baudrate adaptive success, save baudrate\r\n");
        app_user_flash_baudrate_write(g_baudrate);
    }

    if (BAUDRATE_ADAPTIVE_TYPE_BEACON == g_adaptive_type) {
        tal_sw_timer_stop(baudrate_adaptive_event_id);
    }
}

/**
 * @brief baudrate adaptive init
 *
 * @param[in] NULL
 * @return NULL
 */
VOID_T baudrate_adaptive_init(VOID_T)
{
    UINT32_T user_baudrate = app_user_flash_baudrate_read();
    baudrate_adaptive_debug("user flash buadrate %d\r\n", user_baudrate);
    if ((DEFAULT_SUPPORT_BAUDRATE_9600 !=user_baudrate)&&(DEFAULT_SUPPORT_BAUDRATE_115200 !=user_baudrate)) {
        g_get_baudrate_sucess = FALSE;
        return;
    }
    g_get_baudrate_sucess = TRUE;
}

/**
 * @brief baudrate_adaptive event init
 *
 * @param[in] NULL
 * @return NULL
 */
VOID_T baudrate_adaptive_event_init(VOID_T)
{
    tal_sw_timer_create(baudrate_adaptive_event_handler, NULL, &baudrate_adaptive_event_id);
#ifndef TUYA_ZG_ROUTER
    tal_sw_timer_create(check_wakeup_io_event_handler, NULL, &check_wakeup_io_event_id);
#endif
}

