
#include "jsmn.h"
#include "tuya_tools.h"
#include "tuya_jsmn.h"
#include "tkl_memory.h"
#include <string.h>

#ifdef MCU_CORE_8258
#if END_DEVICE
#define TUYA_JSMN_MALLOC    wakup_malloc
#define TUYA_JSMN_FREE      wakup_free
#else
#define TUYA_JSMN_MALLOC	tkl_system_malloc
#define TUYA_JSMN_FREE	    tkl_system_free
#endif
#else
#define TUYA_JSMN_FREE      tkl_system_free
#define TUYA_JSMN_MALLOC    tkl_system_malloc
#endif


#define TUYA_SKD_DBG 1
#if TUYA_SKD_DBG
#include "tkl_uart.h"
#include "tlprintf.h"
#define tuya_jsmn_debug(...)    tkl_dbg_printf(__VA_ARGS__)
#else
#define tuya_jsmn_debug(...)
#endif

OPERATE_RET json_value_to_int(CHAR_T *json_str, jsmntok_t *jstok, INT_T *out_value)
{
    INT_T i = 0; 
    INT_T value = 0;

    if(jstok->type != JSMN_PRIMITIVE) {
        return OPRT_CJSON_PARSE_ERR;
    }

    UINT8_T value_len = jstok->end-jstok->start;

    //one extra byte for \0
    char * value_cache = (char *)TUYA_JSMN_MALLOC(value_len+1);
    if(value_cache == NULL){
        return OPRT_COM_ERROR;
    }

    tkl_system_memset(value_cache, 0, value_len+1);
    tkl_system_memcpy(value_cache,json_str+jstok->start,value_len);
    if(strstr(value_cache, "true")) {
        *out_value = 1;
        TUYA_JSMN_FREE(value_cache);
        value_cache = NULL;
        return OPRT_OK;
    }
    else if(strstr(value_cache, "false")) {
        *out_value = 0;
        TUYA_JSMN_FREE(value_cache);
        value_cache = NULL;
        return OPRT_OK;
    }
    TUYA_JSMN_FREE(value_cache);
    value_cache = NULL;

    for (i=jstok->start; i<jstok->end; i++) {
        if(json_str[i]>='0' && json_str[i]<='9') {
            value = value*10 + (json_str[i]-'0');
        }
        else {
            return OPRT_CJSON_PARSE_ERR;
        }
    }
    *out_value = value;

    return OPRT_OK;
}


jsmntok_t *tuya_json_get_value(CHAR_T *json_str, CHAR_T *key, jsmntok_t *token, UINT8_T token_len)
{
    UINT8_T i;
    INT_T ret = 0;
    INT_T key_len = tuya_strlen(key);
    INT_T token_key_len = 0;
    jsmn_parser parser;

    jsmn_init(&parser);
    ret = jsmn_parse(&parser, json_str, tuya_strlen(json_str), token, token_len);
    if(ret <= 0) {
        return NULL;
    } 

    for (i=0; i<ret; i++) {
        token_key_len = token[i].end - token[i].start;
        if (token[i].type == JSMN_STRING &&
            token[i].size >= 1 &&
            token_key_len == key_len &&
            !tkl_system_memcmp(key, &json_str[token[i].start], key_len)) {
            return &token[i+1];
        }
    }

    return NULL;
}


OPERATE_RET tuya_json_get_int_value(const CHAR_T *json_str, CHAR_T *key, INT_T *out_value)
{
    UINT16_T i = 0;
    OPERATE_RET ret;
    CHAR_T *temp_str = NULL;
    UINT16_T jsnm_token_num = JSMN_OBJ_ITEM_NUMBER*2 + 1;
    jsmntok_t *token = NULL;
    
    token = TUYA_JSMN_MALLOC(sizeof(jsmntok_t)*(JSMN_OBJ_ITEM_NUMBER*2+1));
    if(token){
        tkl_system_memset(token,0x00,(sizeof(jsmntok_t)*(JSMN_OBJ_ITEM_NUMBER*2+1)));
        if(json_str == NULL || key == NULL || out_value == 0) {
            TUYA_JSMN_FREE(token);
            token = NULL;
            return OPRT_INVALID_PARM;
        }

        for(i=0; i<JSMN_OBJ_MAX_LEN; i++) {
            if(json_str[i] == '}') {
                temp_str = (CHAR_T *)json_str;
                break;
            }
        }

        if(!temp_str) {
            TUYA_JSMN_FREE(token);
            token = NULL;
            return OPRT_CJSON_PARSE_ERR;
        }

        jsmntok_t *jstok = tuya_json_get_value(temp_str, key, token, jsnm_token_num);
        if(jstok == NULL) {
            TUYA_JSMN_FREE(token);
            token = NULL;
            return OPRT_CJSON_PARSE_ERR;
        }
        if(token){
            TUYA_JSMN_FREE(token);
            token = NULL;
        }
        ret = json_value_to_int(temp_str, jstok, out_value);
        return ret;
    }
    else{
        tuya_jsmn_debug("malloc fail size:%x\r\n",sizeof(jsmntok_t)*(JSMN_OBJ_ITEM_NUMBER*2+1));
        return OPRT_MALLOC_FAILED;
    }
}

OPERATE_RET tuya_json_get_bool_value(const CHAR_T *json_str, CHAR_T *key, INT_T *out_value)
{
    return tuya_json_get_int_value(json_str, key, out_value);
}

OPERATE_RET tuya_json_get_string_value(const CHAR_T *json_str, CHAR_T *key, CHAR_T *out_buff, UINT16_T buff_len)
{
    UINT16_T i = 0;
    UINT16_T len = 0;
    CHAR_T *temp_str = NULL;
    UINT16_T jsnm_token_num = (JSMN_OBJ_ITEM_NUMBER*2+1);
    jsmntok_t *token = NULL;
    
    token = TUYA_JSMN_MALLOC(sizeof(jsmntok_t)*(JSMN_OBJ_ITEM_NUMBER*2+1));
    if(token){
        tkl_system_memset(token,0x00,(sizeof(jsmntok_t)*(JSMN_OBJ_ITEM_NUMBER*2+1)));
        if(json_str == NULL || key == NULL || 
            out_buff == NULL || buff_len == 0) {
            TUYA_JSMN_FREE(token);
            token = NULL;
            return OPRT_INVALID_PARM;
        }

        for(i=0; i<JSMN_OBJ_MAX_LEN; i++) {
            if(json_str[i] == '}') {
                temp_str = (CHAR_T *)json_str;
                break;
            }
        }

        if(!temp_str) {
            TUYA_JSMN_FREE(token);
            token = NULL;
            return OPRT_CJSON_PARSE_ERR;
        }

        jsmntok_t *jstok = tuya_json_get_value(temp_str, key, token, jsnm_token_num);
        if(jstok == NULL) {
            TUYA_JSMN_FREE(token);
            token = NULL;
            return OPRT_CJSON_PARSE_ERR;
        }

        if(jstok->type != JSMN_STRING) {
            TUYA_JSMN_FREE(token);
            token = NULL;
            return OPRT_CJSON_PARSE_ERR;
        }
        
        len = jstok->end - jstok->start;
        if(len >= buff_len) {
            TUYA_JSMN_FREE(token);
            token = NULL;
            return OPRT_CJSON_PARSE_ERR;
        }

        tkl_system_memcpy(out_buff, temp_str+jstok->start, len);
        out_buff[len] = '\0';
        if(token){
            TUYA_JSMN_FREE(token);
            token = NULL;
        }
        return OPRT_OK;
    }
    else{
        return OPRT_MALLOC_FAILED;
    }

}


